class APIErrors(Exception):
    pass


class UnauthorizedError(APIErrors):
    def __init__(self, message, errors):
        super(UnauthorizedError, self).__init__(message)
        self.errors = errors
        print('Unauthorized Error. Please verify validity of access token.')
        pass

class InternalServerError(APIErrors):
    def __init__(self, message, errors):
        super(InternalServerError, self).__init__(message)
        self.errors = errors
        print('Internal Server Error. Please verify availability of the service.')
        pass



import requests


class Organziations():
    def list_organizations(self):
        url = self.url + 'organizations'
        return self._get_handler(url)

    def show_organization(self, organization):
        url = self.url + 'organizations/{}'.format(organization)
        return self._get_handler(url)

    def update_organization(self, organization_name, email):
        url = self.url + 'organizations/{}'.format(organization_name)
        payload = {
            "data": {
                "type": "organizations",
                "attributes": {
                    "name": "hashicorp",
                    "email": email
                }
            }
        }
        return self._patch_handler(url=url, json=payload)

    def destroy_organization(self, organization_name):
        url = self.url + 'organizations/{}'.format(organization_name)
        return self._delete_handler(url)


    def create_organization(self, organization_name, email):
        url = self.url + 'organizations'
        payload = {
            "data": {
                "type": organization_name,
                "attributes": {
                    "name": "hashicorp",
                    "email": email
                }
            }
        }
        return self._post_handler(url=url, json=payload)




class Variables():
    def create_workspace_variable(self, workspace_id, key, value, category='terraform', hcl=False, sensitive=False):
        url = self.url + 'vars'
        payload = {
            "data": {
                "type": "vars",
                "attributes": {
                    "key": key,
                    "value": value,
                    "category": category,
                    "hcl": hcl,
                    "sensitive": sensitive
                },
                "relationships": {
                    "workspace": {
                        "data": {
                            "id": workspace_id,
                            "type": "workspaces"
                        }
                    }
                }
            }
        }
        return self._post_handler(url=url, json=payload)

    def modify_workspace_variable(self, workspace_id, attributes):
        """
        Modify an existing workspace variable in a given workspace, omit fields to not overwrite them.
        :param workspace_id: String of workspace id for terraform variables
        :param attributes: A map of attributes. i.e.
        "id": workspace_id,
                {
                    "key": "name",
                    "value": "value",
                    "category": "terraform",
                    "hcl": False,
                    "sensitive": False
                }
        """
        url = self.url + 'vars/{}'.format(workspace_id)
        payload = {
            "data": {
                "id": workspace_id,
                "attributes": attributes,
                "type": "vars"
            }
        }

        return self._post_handler(url=url, json=payload)

    def list_variables(self, organization, workspace):
        """
        Lists variables in a given workspace.
        :param organization: String of organization Name
        :param workspace: String of workspace id for terraform variables
        """
        url = self.url + 'vars?filter%5Borganization%5D%5Bname%5D={}&filter%5Bworkspace%5D%5Bname%5D={}'\
            .format(organization, workspace)
        return self._get_handler(url)

    def delete_workspace_variable(self, variable_id):
        """
        Deletes variable for a given variable id.
        :param variable_id: string, variable id found via list variables
        """
        url = self.url + '/vars/{}'.format(variable_id)
        return self._delete_handler(url)



from json import loads
import requests


class Workspaces():
    def list_workspaces(self, organization):
        url = self.url + 'organizations/{}/workspaces'.format(organization)
        return self._get_handler(url)

    def create_workspace(self, organization, workspace_name):
        url = self.url + 'organizations/{}/workspaces'.format(organization)
        payload = {
            "data": {
                "attributes": {
                    "name": workspace_name
                },
                "type": "workspaces"
            }
        }
        return self._post_handler(url=url, json=payload)

    def delete_workspace(self, organization, workspace_name):
        url = self.url + 'organizations/{}/workspaces/{}'.format(organization, workspace_name)
        return self._delete_handler(url)

    def list_workspace_ids(self, organization):
        return [str(workspaces["id"]) for workspaces in loads
                (self.list_workspaces(organization))["data"]]

    def get_workspace_non_confirmed_runs(self, workspace_id):
        return [runs for runs in loads(self.get_workspace_runs(workspace_id))["data"] if
                runs["attributes"]["actions"]["is-confirmable"]]

    def get_workspace_current_state_version(self, workspace_id):
        url = self.url + 'workspaces/{}/current-state-version'.format(workspace_id)
        return self._get_handler(url)

    def get_workspace_current_statefile(self, workspace_id):
        try:
            url = loads(
                self.get_workspace_current_state_version(workspace_id)
            )["data"]["attributes"]["hosted-state-download-url"]
        except KeyError:
            return
        return requests.get(url=url).content

    def show_workspace(self, organization, workspace_name):
        url = self.url + 'organizations/{}/workspaces/{}'.format(organization, workspace_name)
        return self._get_handler(url)

    def update_workspace(self, update_params, organization):
        """
        :param organization: Organization of workspace
        :param update_params: Parameters to update, omit fields to not alter them. i.e.
        {
        "name": "test-workspace",
        "terraform_version": "0.12.1",
        "working-directory": "test/awesome-directory",
        "vcs-repo": {
            "identifier": "github/Terraform-Testing",
            "branch": "test",
            "ingress-submodules": False,
            "oauth-token-id": "ot-XXXXXXXXX"
            }
        }
        """
        url = self.url + 'organizations/{}/workspaces/{}'.format(organization, update_params["name"])
        payload = {
            "data": {
                "attributes": update_params,
                "type": "workspaces"
            }
        }
        return self._patch_handler(url=url, json=payload)
